; Title - BAVE (Bally Astrocade Videogame Engine) / Template
; By    - Michael Garber
;		- 0.10 - Sept 10, 2011 - Final Code cleanups for release
;		- 0.08 - Sept 9, 2011 - RANGED routine no longer called
;							  - No more bios routines, moved stack to $4fff (saved another $30)
;		- 0.07 - Sept 8, 2011 - bios system calls for setting video registers replaced
;							  - All ram gets wiped at start
;							  - GameState/Custom Logic hook added
;							  - Moved Stack to $4fd0 (saved $30 bytes of ram!!)
;		- 0.06 - Sept 8, 2011 - Added riffraffgames splash screen
;		- 0.05 - Sept 7, 2011 - Added DrawString for displaying text
;							  - custom clearscreen routine used now instead of bios
;		- 0.04 - Sept 7, 2011 - Added ability to use custom/bank switched raster int code
;		- 0.03 - Sept 6, 2011 - Added font for drawing text, added DrawLetter routine
;		- 0.02 - Sept 5, 2011 - Debugging routines added (memory browser/showbyte)
;		- 0.01 - Sept 4, 2011 - Project started
;
; About this Program
; ------------------ 
; This is the main BAVE engine and it represents a template for starting new 256k projects
; for the Bally Astrocade using Steve Tucker's 256k bank-switch cart

INCLUDE "HVGLIB.H"			; Home Video Game Library Header
INCLUDE	"bave.h"			; include file for bave routines

; options: _NONE _GAMETICK _SHOWBYTE
	DSTATE_DEFAULT 		EQU		DSTATE_GAMETICK
	GameVertLines	EQU	90		; # vertical lines visible for game
	StartStack		EQU	$4fff	; leave $20 bytes for stack ($4fa8-$4fff)
								; also if you want to use BIOS routine, ram above $4fc0 is
								; used by them, depending on the routine.
; variable ram alloc in bave.h

	ORG	FIRSTC			; First byte of Cartridge
	DB	$55				; User Cartridge Sentinel

; Menu Selection Choice #1
	DW	MENUST			; Next Menu Link
	DW	ProgramName		; Address of Program Name text
	DW	BaveStart		; Jump here if selected
; --------------------------------------------------------------------------------------------
; ** WARNING ** The routine below is expected to be in this exact memory location so returning
;               raster code from modules will know where it is. (Do not move it)
; --------------------------------------------------------------------------------------------
EndOfCustomRaster:	; $2007
	LD	A,(LastSetBank)
	LD	L,A
	LD	H,0
	LD	BC,$3FC0
	ADD	HL,BC
	LD	A,(HL)  ; Do the bank switch by reading this addy

	POP	IX
	POP	HL
	POP	BC
	POP AF
	EI
	RET
; ----------------------------------------------------------------
; Program Start
; ----------------------------------------------------------------
BaveStart:
    LD	SP,StartStack  	; $4fa8-$4fd0 $28 bytes
	; wipe entire ram area
	XOR	A
	LD	A,$00
	LD	($4000),A		; byte to fill memory with
	LD	HL,$4000
	LD	DE,$4001
	LD	BC,$0fff
	LDIR
	CALL InitBave
	CALL InitScreenStuff
	EI

; **************************
; --------------------------
; ----- MAIN GAME LOOP -----
; --------------------------
; **************************
GameLoop:
WaitForVB:
	LD	A,(VBlankHit)
	AND	00000001B
	JR	Z,WaitForVB
	XOR	A
	LD	(VBlankHit),A
; vblank hit - start workload
	CALL RandomNum
	
	LD	IX,GameTick		; bump GameTick
	INC	(IX)
	JR	NZ,GameL1
	INC	(IX+1)
GameL1:

	CALL DebounceTriggers	; maintain trigger values
	CALL GameLogic			; once-a-vblank code
	CALL DebugLogic			; display any debug information
	JR	GameLoop
; ----------------------------------------------------------------
GameLogic:
	LD	A,(CustBank)		; custom logic?
	BIT	LOGIC_CUSTOM,A
	JR	Z,StandardLogic
	AND	01111111B		; get bank of logic
	LD	L,A
	LD	H,0
	LD	BC,$3FC0
	ADD	HL,BC
	LD	A,(HL)  ; Switch banks to where the logic is
	LD	HL,CustReturn
	PUSH HL		; return to code below to reset bank
	LD	HL,(CustLogic)
	JP (HL)
CustReturn:
	LD	A,(LastSetBank)	; return bank to what it was before
	LD	L,A
	LD	H,0
	LD	BC,$3FC0
	ADD	HL,BC
	LD	A,(HL)  ; bank switch
	RET

StandardLogic:
	XOR	A
	CALL TestDTrigger
	JR	Z,StdLogicEnd
	CALL ClearDTrigger

	; advance state
	LD	A,(GameState)
	CP	GSTATE_INTRO
	JR	NZ,StdLogicEnd
	LD	A,GSTATE_SPLASH
	LD	(GameState),A

	LD	A,STARTBANK
	CALL SetCartBank
	CALL $3000	; call code to initialize this module

StdLogicEnd:
	RET
; ----------------------------------------------------------------
InitBave:
	LD	IX,DebugColors
	CALL SetColorTable

	XOR	A
	LD 	(GameTick),A
	LD	(GameTick+1),A
	LD	(CustBank),A		; no custom logic yet

	LD	A,$aa				; Seed the random number
	LD	(TickRND),A
	LD	(TickRND+1),A
	
	LD	A,STARTBANK
	CALL SetCartBank
	LD	A,GSTATE_INTRO
	LD	(GameState),A
	LD	A,DSTATE_DEFAULT
	LD	(DebugState),A

	; display bave welcome screen
	LD	IX,ProgramName
	LD	HL,$4000
	CALL DrawString

	LD	IX,EngineSizeLabel
	CALL DrawString
; display last byte of main engine (free space)
	LD	A,EndOfMainModule>>8
	CALL DisplayHex
	LD	A,EndofMainModule & $ff
	CALL DisplayHex

	LD	IX,ModSizeLabel
	CALL DrawString
	; module space used
	CALL ShowModuleSpace
	RET
EngineSizeLabel:
	DB '   ENGINE:',$00
ModSizeLabel:
	DB '   MOD:',$00
; ----------------------------------------------------------------
; enter with: A = bank to switch to
SetCartBank:
	PUSH	AF
	PUSH	BC
	PUSH	HL
	
	LD	(LastSetBank),A
	LD	L,A
	LD	H,0
	LD	BC,$3FC0
	ADD	HL,BC
	LD	A,(HL)  ; Bank switch by reading this addy

	POP	HL
	POP	BC
	POP AF
	RET
; ----------------------------------------------------------------
ShowModuleSpace:
	LD	A,(MOD_MODLEN+1)
	CALL DisplayHex
	LD	A,(MOD_MODLEN)
	JP DisplayHex
; ----------------------------------------------------------------
; This random number generator can be improved
RandomNum:
	LD	HL,(TickRND)
	LD	A,R
	LD	D,A
	LD	E,B
	ADD	HL,DE
	ADD	A,L
	XOR	H
	LD 	(TickRND),HL
	RET
; ----------------------------------------------------------------
ClearScreen:
	XOR	A
	LD	A,$00
	LD	($4000),A		; byte to fill memory with
	LD	HL,$4000
	LD	DE,$4001
	LD	BC,40*GameVertLines-1
	LDIR
	RET
; ----------------------------------------------------------------
ScrollScreenUp:
	LD	HL,$4000
	LD	DE,$4000+GameVertLines*40
	LD	BC,160
	LDIR

	LD	HL,$4000+160
	LD	DE,$4000
	LD	BC,GameVertLines*40
	LDIR
	RET
; ----------------------------------------------------------------
ScrollScreenDown:
	LD	HL,$4000+GameVertLines*40-1
	LD	DE,$4000+GameVertLines*40-1+160
	LD	BC,GameVertLines*40
	LDDR

	LD	HL,$4000+GameVertLines*40-1+160
	LD	DE,$4000+159	; copy bottom line to top
	LD	BC,160
	LDDR
	RET
; ----------------------------------------------------------------
; clear debounce trigger (a = controller #)
ClearDTrigger:
	LD	HL, DebTrigs
	CP	0
	JR	NZ,CTNotZero
	RES	0,(HL)
	RET
CTNotZero:
	CP	1
	JR	NZ,CTNotOne
	RES	1,(HL)
	RET
CTNotOne:
	CP	2
	JR	NZ,CTNotTwo
	RES	2,(HL)
	RET
CTNotTwo:
	RES	3,(HL)
	RET
; ----------------------------------------------------------------
; test debounce trigger (a = controller #)
TestDTrigger:
	LD	HL, DebTrigs
	CP	0
	JR	NZ,TDNotZero
	BIT	0,(HL)
	RET
TDNotZero:
	CP	1
	JR	NZ,TDNotOne
	BIT	1,(HL)
	RET
TDNotOne:
	CP	2
	JR	NZ,TDNotTwo
	BIT	2,(HL)
	RET
TDNotTwo:
   	BIT	3,(HL)
	RET
; ----------------------------------------------------------------
; read joystick triggers, set proper bits in DebTrig
; this needs to check trig 2-4
DebounceTriggers:
	LD	HL, DebTrigs
	IN	A,(SW0)			; Get joystick info from controller 1
	BIT	CHTRIG, A	 	; test trigger
	JR	NZ, DT1Set
; trigger not currently held down
	RES	4,(HL)
	RES	0,(HL)
	JR	DT1Cont
; trigger held down
DT1Set:
	BIT 4,(HL)			; waiting to clear bit?
	JR	NZ,DT1Cont
	SET	4,(HL)
	SET	0,(HL)
DT1Cont:
	RET
; ----------------------------------------------------------------
AnyTrigger:
	IN	A,(SW0)			; Get joystick info from controller 1
	LD	B,A
	IN	A,(SW1)			; Get joystick info from controller 2
	OR	B
	LD	B,A
	IN	A,(SW2)			; Get joystick info from controller 3
	OR	B
	LD	B,A
	IN	A,(SW3)			; Get joystick info from controller 4
	OR	B
	RET
; ----------------------------------------------------------------
; init screen registers
InitScreenStuff:
	; set video information
	LD	A,GameVertLines*2
	OUT	(VERBL),A		; vertical blank line
	LD	A,39
	OUT	(HORCB),A		; horizontal color boundry
	LD	A,$08
	OUT	(INMOD),A		; interrupt mode (raster interrupt on)
	
;	XOR	A
;	LD	(RasterBank),A		; turn off custom raster
	LD	A,1<<RAST_CUSTOM
	LD	(RasterBank),A		; bank 0, custom raster
	LD	A,CustomRasters>>8
	LD	(RasterCode+1),A
	LD	A,CustomRasters & $ff
	LD	(RasterCode),A

	IM2
	LD A,SIVEC>>8		; set addy of vector for int
	LD I,A
	LD A,SIVEC & $ff
	OUT (INFBK), A

;	LD	A,(DebugBreaks)		; if not using custom raster
	LD	A,(CustomBreaks)
	OUT (INLIN), A
	XOR	A
	LD	(CurrentIntLine),A
	RET
; ----------------------------------------------------------------
; Screen interrupt
SIVEC	DW	ScreenInt
ScreenInt:
	PUSH AF
	PUSH BC
	PUSH HL
	PUSH IX

	; check bit set for custom raster code, if set jump there instead
	LD	IX,RasterBank
	LD	A,(IX)
	BIT	RAST_CUSTOM,A
	JR	Z,MonRasters		; use monitor (default) raster code
	; get address of custom code and jump there
	AND	01111111B		; get bank
	LD	L,A
	LD	H,0
	LD	BC,$3FC0
	ADD	HL,BC
	LD	A,(HL)  ; Do the bank switch

;	LD	IX,RasterCode
;	LD 	A,(IX)
;	LD	L,A
;	LD	A,(IX+1)
;	LD	H,A

	LD	HL,(RasterCode)
	JP	(HL)
; ----------------------------------------------------------------
CustomBreaks:
	DB	181		; bottom of screen was 186
	DB	9		;
	DB	160		;
; ----------------------------------------------------------------
CustomBands:
	DB	$01 ; grey
	DB	$00	; black
	DB	$a9	; green
; ----------------------------------------------------------------
CustomRasters:
	LD	A,(CurrentIntLine)
	LD	C,A
	LD	B,0
	LD	HL,CustomBands
	ADD	HL,BC
	LD	A,(HL)	; get color value for left color band
	OUT  (COL0L),A		; Right Color Register 0

	LD	A,(CurrentIntLine)
	CP	0
	JR	NZ,CustR1
; first line in table, placed at bottom of screen in vblank area
	LD	A,1
	LD (VBlankHit),A	; signal "vblank" to sync main game code

CustR1:
	; get to next line
	LD	A,(CurrentIntLine)
	INC A
	CP	3
	JR	C,CustR2
	XOR	A
CustR2:
	LD	(CurrentIntLine),A
	LD	C,A
	LD	B,0
	LD	HL,CustomBreaks
	ADD	HL,BC
	LD	A,(HL)			; get next line to interrupt on
	OUT (INLIN),A		; ($0f)
	JP	RASTEND
; ----------------------------------------------------------------
BankNoStr:
	DB	"BANK:",$00
; ----------------------------------------------------------------
DebugBreaks:
	DB	181		; bottom of screen
	DB	9		;
	DB	93		;
; ----------------------------------------------------------------
DebugBands:
	DB	$51,$51	; red/red
	DB	$fa,$00	; blue/black
	DB	$a9,$00	; green/black
; ----------------------------------------------------------------
MonRasters:
	LD	A,(CurrentIntLine)
	ADD	A,A
	LD	C,A
	LD	B,0
	LD	HL,DebugBands
	ADD	HL,BC
	LD	A,(HL)	; get color value for left color band
	OUT  (COL0L),A		; Right Color Register 0
	INC	HL
	LD	A,(HL)	; get color value for right color band
	OUT  (COL0R),A		; Right Color Register 0

	LD	A,(CurrentIntLine)
	CP	0
	JR	NZ,MonRaster1
; first line in table, placed at bottom of screen in vblank area
	LD	A,1
	LD (VBlankHit),A	; signal "vblank" to sync main game code
	LD	A,5			; split to the right of the addresses
	OUT	(HORCB),A

MonRaster1:
	; get to next line
	LD	A,(CurrentIntLine)
	INC A
	CP	3
	JR	C,MonRaster2
	XOR	A
MonRaster2:
	LD	(CurrentIntLine),A
	LD	C,A
	LD	B,0
	LD	HL,DebugBreaks
	ADD	HL,BC
	LD	A,(HL)			; get next line to interrupt on
	OUT (INLIN),A		; ($0f)

;SKP3:
	POP	IX
	POP	HL
	POP	BC
	POP AF
	EI
	RET
; ----------------------------------------------------------------
DebugLogic:
; debug = emergency label print
;	LD	HL,$4000+84*40+36	; 152,84
;	LD	A,DebugState>>8
;	CALL DisplayHex
;	LD	A,DebugState & $ff
;	CALL DisplayHex
;	RET

	LD	A,(DebugState)		; what debug information to show?
	CP	0
	JR	NZ,DebugIsOn
	RET

DebugIsOn:
	LD	HL,$4000+84*40+36	; 152,84 (bottom-right of screen)
	CP	DSTATE_SHOWBYTE
	JR	NZ,DebugLogic2
	; debug numbers bottom-right of screen
	LD	A,(ShowB1)
	CALL DisplayHex
	LD	A,(ShowB2)
	JR DisplayHex

DebugLogic2:
	CP	DSTATE_BROWSE
	JR	NZ,DebugLogic3

	CALL BrowseLogic

	LD	HL,$4000
	LD	A,(DebugAddr+1)
	LD	(DebugTemp+1),A
	CALL DisplayHex
	LD	A,(DebugAddr)
	LD	(DebugTemp),A
	CALL DisplayHex

	INC	HL
	IN	A,(POT0)		; Get Pot 0 value (Player 1 knob)
	XOR	$ff
	CALL DisplayHex
	INC	HL
	CALL DrawLetter

	LD	HL,$4000+28
	LD	IX,BankNoStr
	CALL DrawString
	IN	A,(SW0)			; Get joystick info from controller 1
	BIT	JoyTrig,A
	JR	Z,DLSkipBankSet
	IN	A,(POT0)		; Get Pot 0 value (Player 1 knob)
	XOR	$ff
	AND	11111100B
	RRCA
	RRCA
	CALL SetCartBank
	CALL DisplayHex
	INC	HL
	CALL ShowModuleSpace

DLSkipBankSet:
	LD	HL,$4000+6*40	; 0,5
	LD	B,14
BrowseLoop1:
	CALL BrowseLine
	DJNZ BrowseLoop1
	RET
	
DebugLogic3:

DebugDefault:
	LD	A,(GameTick+1)
	CALL DisplayHex
	LD	A,(GameTick)
	; fall thru to DisplayHex
; ----------------------------------------------------------------
; A = byte to display, HL = screen address where to draw
; HL gets moved to right of drawn 2 digits
DisplayHex:
	PUSH AF
	PUSH BC
	PUSH DE

	PUSH AF				; save A for lower nibble
	AND	$f0				; need high nibble only
	RRCA
	RRCA
	RRCA
	RRCA
	CALL	DrawDigit
	POP	AF
	AND	$0f				; need low nibble only
	CALL	DrawDigit

	POP DE
	POP BC
	POP	AF
    RET
; ----------------------------------------------------------------
; HL = screen destination address, A = digit
; moves HL to point to where next digit should be drawn (4 pixels right)
DrawDigit:
	PUSH IX

	LD	B,A
	ADD	A,A	; *2
	ADD	A,A	; *4
	ADD	A,B	; *5	(5 bytes per number gfx)
	LD	IX,Num0Gfx
	LD	C,A
	LD	B,0
	ADD	IX,BC
	
	LD DE,40		; amount to add to get to next row in graphic data
	LD A,(IX)		; copy the 5 bytes of digit (not looping for speed)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+1)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+2)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+3)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+4)
	LD (HL),A

	XOR	A		; clear carry
	LD BC,40*4-1
	SBC HL,BC	; move HL to next tile on right

	POP	IX
	RET
; ----------------------------------------------------------------
; A = byte to display, B = mask, HL = screen address where to draw
; HL gets moved to right of drawn 2 digits
DisplayHexMasked:
	PUSH AF
	PUSH BC
	PUSH DE

	PUSH AF				; save A for lower nibble
	AND	$f0				; need high nibble only
	RRCA
	RRCA
	RRCA
	RRCA
	CALL	DrawDigitMasked
	POP	AF
	AND	$0f				; need low nibble only
	CALL	DrawDigitMasked

	POP DE
	POP BC
	POP	AF
    RET
; ----------------------------------------------------------------
; HL = screen destination address, A = digit, B = mask
; moves HL to point to where next digit should be drawn (4 pixels right)
DrawDigitMasked:
	PUSH IX
	PUSH BC
	
	PUSH BC
	LD	B,A
	ADD	A,A	; *2
	ADD	A,A	; *4
	ADD	A,B	; *5	(5 bytes per number gfx)
	LD	IX,Num0Gfx
	LD	C,A
	LD	B,0
	ADD	IX,BC
	POP	BC

	LD DE,40		; amount to add to get to next row in graphic data
	LD A,(IX)		; copy the 5 bytes of digit (not looping for speed)
	AND	B
	LD (HL),A
	ADD HL,DE
	LD A,(IX+1)
	AND	B
	LD (HL),A
	ADD HL,DE
	LD A,(IX+2)
	AND	B
	LD (HL),A
	ADD HL,DE
	LD A,(IX+3)
	AND	B
	LD (HL),A
	ADD HL,DE
	LD A,(IX+4)
	AND	B
	LD (HL),A

	XOR	A
	LD BC,40*4-1
	SBC HL,BC	; move HL to next tile on right

	POP	BC
	POP	IX
	RET
; ----------------------------------------------------------------
; add some params here.
InitDebugBrowse:
	LD	A,DSTATE_BROWSE
	LD	(DebugState),A

	LD	A,$4f		; start at $4f20 (displays thru end of ram)
	LD	(DebugAddr+1),A
	LD	A,$20
	LD	(DebugAddr),A

	XOR	A
	LD	(CustBank),A		; no custom logic for debug
	DI
	LD	(RasterBank),A		; turn off custom raster
	LD	IX,DebugColors
	CALL SetColorTable
	EI

	JP ClearScreen
; ----------------------------------------------------------------
; Color Table for Debug (greyscale/misc)
DebugColors:
	DB     $07        ; Color 3 Left white
	DB     $05        ; Color 2 Left med grey
	DB     $02        ; Color 1 Left dk grey
	DB     $00        ; Color 0 Left black
	DB     $07        ; Color 3 Right
	DB     $05        ; Color 2 Right white
	DB     $20        ; Color 1 Right green
	DB     $00        ; Color 0 Right Black
; ----------------------------------------------------------------
; Enter with: IX = table of 8 bytes (color data 3L 2L 1L 0L 3R 2R 1R 0R)
SetColorTable:
	LD	A,(IX+0)
	OUT  (COL3L),A
	LD	A,(IX+1)
	OUT  (COL2L),A
	LD	A,(IX+2)
	OUT  (COL1L),A
	LD	A,(IX+3)
	OUT  (COL0L),A

	LD	A,(IX+4)
	OUT  (COL3R),A
	LD	A,(IX+5)
	OUT  (COL2R),A
	LD	A,(IX+6)
	OUT  (COL1R),A
	LD	A,(IX+7)
	OUT  (COL0R),A
	RET
; ----------------------------------------------------------------
BrowseLogic:
	LD	IX,DebugAddr	; home debug addr
	LD	A,(IX);
	LD	L,A
	LD	A,(IX+1)
	LD	H,A

	LD	BC,$10			; amount to change dataaddr

	IN	A,(SW0)			; Get joystick info from controller 1
	BIT	JoyUP,A
	JR	Z,BLNotUp
	; change debugaddy
	SBC	HL,BC
	JR	BLSaveDest

BLNotUp:
	BIT	JoyDown,A
	JR	Z,BLNotDown
	ADD	HL,BC
	JR	BLSaveDest

BLNotDown:
	LD	BC,$100			; amount to change dataaddr
	BIT	JoyLeft,A
	JR	Z,BLNotLeft
	; change debugaddy
	SBC	HL,BC
	JR	BLSaveDest

BLNotLeft:
	BIT	JoyRight,A
	JR	Z,BLNotRight
	ADD	HL,BC
	JR	BLSaveDest

BLNotRight:
	JR	BLExit

BLSaveDest:	
	LD	(IX),L
	LD	(IX+1),H
BLExit:
	RET
; ----------------------------------------------------------------
BrowseLine:
	PUSH BC

	LD	IX,DebugTemp
	LD	A,(IX+1)
	CALL DisplayHex
	LD	A,(IX)
	CALL DisplayHex

	CP	$c0				; $c0 for 256k, $80 if 512k cart
	JR	C,BLLoopOK		; low byte < $c0
	LD	A,(IX+1)
	CP	$3f
	JR	NZ,BLLoopOK
	; this ram is forbidden to read, display darkened 0's
	LD	BC,$10
	ADD	IY,BC;			; move past this data
	LD	B,18
BLLoopBAD:			; forbidden memory reads here, show bad 0's
	PUSH	BC
	XOR	A
	LD	B,0	; mask it all
	CALL DisplayHexMasked
	POP	BC
	DJNZ BLLoopBAD
	JR	BLDoneWithRow

BLLoopOK:
	EX	DE,HL			; set IY to point where DebugTemp is pointing
	LD	A,(IX)
	LD	L,A
	LD	H,(IX+1)
	PUSH HL
	POP	IY
	EX	DE,HL

	LD	B,4
BLLoop1:
	PUSH BC
	INC	HL

	LD	A,(IY)
	INC	IY
	CALL DisplayHex

	LD	A,(IY)
	INC	IY
	LD	B,10101010B
	CALL DisplayHexMasked

	LD	A,(IY)
	INC	IY
	CALL DisplayHex

	LD	A,(IY)
	INC	IY
	LD	B,10101010B
	CALL DisplayHexMasked

	POP	BC
	DJNZ BLLoop1

BLDoneWithRow:	
	LD	BC,40*6-40
	ADD	HL,BC

	LD	A,(IX)
	ADD	A,$10
	LD	(IX),A
	LD	A,(IX+1)
	ADC	A,0
	LD	(IX+1),A

	POP BC
	RET
; ----------------------------------------------------------------
INCLUDE numfont.h
; ----------------------------------------------------------------
; HL = screen destination address, A = char to draw
; moves HL to point to where next digit should be drawn (4 pixels right)
DrawLetter:
	PUSH AF
	PUSH IX

	CP	94
	JR	NC,DLSkp0	; above last known ascii char, show space
	CP	33			; <= space? non viewable char
	JR	NC,DLSkp1
DLSkp0
	LD	IX,BlankChar	; space char
	JR	DLSkp2
	
DLSkp1:
	LD	IX,font4
	SUB	33			; font starts with ! (33) as the first char
	LD	B,A
	ADD	A,A	; *2
	ADD	A,A	; *4
	ADD	A,B	; *5	(5 bytes per letter gfx)
	LD	C,A
	LD	B,0
	JR	NC,DLSkp3
	INC	B
DLSkp3:
	ADD	IX,BC
	
DLSkp2:
	LD DE,40		; amount to add to get to next row in graphic data
	LD A,(IX)		; copy the 5 bytes of digit (not looping for speed)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+1)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+2)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+3)
	LD (HL),A
	ADD HL,DE
	LD A,(IX+4)
	LD (HL),A

	LD BC,40*4-1
	XOR	A
	SBC HL,BC	; move HL to next tile on right

	POP	IX
	POP AF
	RET

BlankChar:
	DB	00000000B
	DB	00000000B
	DB	00000000B
	DB	00000000B
	DB	00000000B
; ----------------------------------------------------------------
DrawString:
	LD	A,(IX)
	CP	0
	JR	Z,DrawStrDone
	CALL DrawLetter
	INC	IX
	JR	Drawstring
DrawStrDone:
	RET
; ----------------------------------------------------------------
INCLUDE font4.h
; -----------------------------------------------------------------

; Program Name
ProgramName:	DB	'BAVE 0.10',$00

EndOfMainModule:
	DB 0

	ORG	$2ffc	; pad to full 4k bank with engine jump table
	DW	TestDTrigger
	DW	InitDebugBrowse
; End of Program
